import {Ref, ref} from "vue";
import {Models} from "./Models.ts";
import {YiXueLogic} from "./YiXueLogic.ts";


export namespace DataCenter {

    export let debugMode: boolean = false

    export let shengShi: Map<string, string[]> = new Map()

    export let shiQu: Map<string, string[]> = new Map()

    export let quData: Map<string, Models.Area> = new Map()

    let baziRequest: Models.BaZiRequest

    let bazi: Ref<Models.BaZi> = ref()

    let users: Ref<Models.SaveUser[]>

    export let currentFangAnOfTianGan = ref<Models.SiZhu>(null)
    export let currentFangAnOfDiZhi = ref<Models.SiZhu>(null)

    export function getBaZi() {
        if (bazi.value == null) {
            bazi.value = new Models.BaZi()
        }
        return bazi
    }

    export function getBaZiRequest() {
        if (baziRequest == null) {
            baziRequest = new Models.BaZiRequest()
        }
        return baziRequest
    }

    export function setUsers(data: Models.SaveUser[]) {
        if (users == null) {
            users = ref([])
        }
        users.value = data
    }

    export function clearUsers() {
        if (users != null) {
            users.value = []
        }
    }

    export function getDebugMode() {
        return debugMode
    }

    export function getUsers() {
        if (users == null) {
            users = ref([])
        }
        let lst = users.value.sort(function (a, b) {
            if (a.joinTime > b.joinTime) {
                return -1
            } else if (a.joinTime < b.joinTime) {
                return 1
            } else {
                return 0
            }
        })
        return lst
    }

    export function searchArea(pinyin: string) {
        let lst = []
        if (pinyin.indexOf("*") != -1 || pinyin.indexOf("?") != -1) {
            return lst
        }
        let exp: RegExp = new RegExp(pinyin, "g")
        quData.forEach(function (value) {
            if (exp.test(value.shiPinYin)) {
                lst.push(value)
            }
            if (exp.test(value.shiPinYin + value.quPinYin)) {
                lst.push(value)
            }
        })
        return lst
    }

    export function addAreaData(data: Models.Area) {
        let key1 = data.sheng
        if (!shengShi.has(key1)) {
            shengShi.set(key1, [])
        }
        let lst1 = shengShi.get(key1)
        if (lst1.indexOf(data.shi) == -1) {
            lst1.push(data.shi)
        }

        let key2 = data.sheng + data.shi
        if (!shiQu.has(key2)) {
            shiQu.set(key2, [])
        }
        let lst2 = shiQu.get(key2)
        if (lst2.indexOf(data.qu) == -1) {
            lst2.push(data.qu)
        }

        let key3 = data.sheng + data.shi + data.qu
        quData.set(key3, data)
    }

    export function getSiZhuInfo() {
        DataCenter.currentFangAnOfTianGan.value = null
        DataCenter.currentFangAnOfDiZhi.value = null
        YiXueLogic.getSiZhuInfo(getBaZiRequest(), function (dt: Models.BaZi) {
            getBaZi().value = dt
            console.log("local:", dt)
        })
    }

    export function randomSiZhu() {
        let dt = getBaZi().value
        dt.getSiZhus().forEach(function (sz) {
            if (sz.isTianGan) {
                sz.text = YiXueLogic.randomOfTianGan()
            } else {
                sz.text = YiXueLogic.randomOfDiZhi()
            }
        })
        getBaZiRequest().texts = dt.getTexts()
    }


    export function calcCiTiaos() {

        return null
    }

}