export namespace Models {

    function buildString(...strs: string[]) {
        let result = ""
        strs.forEach(function (value) {
            result += value + " "
        })
        return result
    }

    export enum SiZhuWeiZhi {
        extra = -1,
        unknown = 0,
        niangan = 1,
        yuegan = 2,
        rigan = 3,
        shigan = 4,
        nianzhi = 5,
        yuezhi = 6,
        rizhi = 7,
        shizhi = 8,
        nianzhu = 11,
        yuezhu = 12,
        rizhu = 13,
        shizhu = 14,
    }

    export class Area {
        sheng: string
        shi: string
        qu: string
        weidu: number
        jingdu: number
        hour: number
        minute: number
        second: number
        shiPinYin: string
        quPinYin: string
    }

    export class UserInfo {
        date: string
        startDate: string
        location: string
        sheng: string
        shi: string
        qu: string
        jingdu: number
        weidu: number
        qiankun: string
    }

    export class CustomLogin {
        user: string
        psd: string

        toData() {
            return beanToObject(this)
        }
    }

    export class SaveUser {
        id: number
        name: string
        beizhu: string
        isMan: boolean
        leibie: number
        year: number
        month: number
        day: number
        hour: number
        minute: number
        sheng: string
        shi: string
        qu: string
        niangan: string
        nianzhi: string
        yuegan: string
        yuezhi: string
        rigan: string
        rizhi: string
        shigan: string
        shizhi: string
        customer: string
        joinTime: string

        toData() {
            return beanToObject(this)
        }
    }

    export class QueryUser {
        customer: string
        filter: string

        toData() {
            return beanToObject(this)
        }
    }

    export class DeleteUser {
        customer: string
        id: number

        toData() {
            return beanToObject(this)
        }
    }

    export class BaZiRequest {
        /**1：用八字请求信息，2:用时间地点请求信息*/
        mode: number
        /**顺序是先干再支*/
        texts: string[]
        datetime: string
        startDate: string
        location: string
        sheng: string
        shi: string
        qu: string
        weidu: number
        jingdu: number
        qiankun: string

        constructor() {
            this.mode = 1
            this.texts = ["甲", "甲", "甲", "甲", "子", "子", "子", "子"]
            this.datetime = ""
            this.startDate = ""
            this.location = ""
            this.sheng = ""
            this.shi = ""
            this.qu = ""
            this.weidu = 0
            this.jingdu = 0
            this.qiankun = "乾"
        }

        toData() {
            return beanToObject(this)
        }
    }

    export class MuKu {
        text: string
        state: string
        lock: string
        isLock: boolean
        chong: string
        gonghe: string
        zixing: string
        sanhui: string
        anhe: string
        hasChong: boolean
        hasGongHe: boolean
        hasZiXing: boolean
        haiSanHui: boolean
        hasAnHe: boolean
        isOpen: boolean
    }

    export class Score {
        key: string
        he: number
        anHe: number
        chong: number
        chuan: number
        po: number
        total: number
    }

    export class PingJia {
        mubiao: Score[]
        genji: Score[]
        zhuli: Score[]
        mubiaoScore: number
        genjiScore: number
        zhuliScore: number
        shouhuScore: number
        lushenScore: number
        totalScore: number
    }

    export class SiZhu {
        qiankun: string
        text: string = ""
        wuXing: string = ""
        shiShen: string = ""
        yinYang: string = ""
        isTianGan: boolean = false
        weiZhi: SiZhuWeiZhi = SiZhuWeiZhi.unknown
        cangGan1: string
        cangGan2: string
        cangGan3: string
        gan1WuXing: string
        gan2WuXing: string
        gan3WuXing: string
        gan1ShiShen: string
        gan2ShiShen: string
        gan3ShiShen: string
        tongzhuZhangSheng: string
        riyuanZhangSheng: string
        gaonengs: GaoNeng[]
        power: number
        selfPower: number
        muku: MuKu
        isLuShen: boolean
        isBanLu: boolean
        muPower: number
        huoPower: number
        tuPower: number
        jinPower: number
        shuiPower: number
        biKuMuPower: number
        biKuHuoPower: number
        biKuTuPower: number
        biKuJinPower: number
        biKuShuiPower: number
        kaiKuMuPower: number
        kaiKuHuoPower: number
        kaiKuTuPower: number
        kaiKuJinPower: number
        kaiKuShuiPower: number
        extraGaoNengMap: Map<number, GaoNeng[]>
        pingjia: PingJia

        constructor(text: string, isTianGan: boolean) {
            this.text = text
            this.isTianGan = isTianGan
        }

        setData(sz: SiZhu) {
            copyObjectValues(this, sz)
        }

        isQian() {
            return this.qiankun == "乾"
        }

        setText(text: string) {
            let result = false
            if (this.text != text) {
                this.text = text
                result = true
            }
            return result
        }

        getWeiZhiAsZhu() {
            let result = Models.SiZhuWeiZhi.unknown
            switch (this.weiZhi) {
                case Models.SiZhuWeiZhi.niangan:
                case Models.SiZhuWeiZhi.nianzhi:
                    result = Models.SiZhuWeiZhi.nianzhu
                    break
                case Models.SiZhuWeiZhi.yuegan:
                case Models.SiZhuWeiZhi.yuezhi:
                    result = Models.SiZhuWeiZhi.yuezhu
                    break
                case Models.SiZhuWeiZhi.rigan:
                case Models.SiZhuWeiZhi.rizhi:
                    result = Models.SiZhuWeiZhi.rizhu
                    break
                case Models.SiZhuWeiZhi.shigan:
                case Models.SiZhuWeiZhi.shizhi:
                    result = Models.SiZhuWeiZhi.shizhu
                    break
            }
            return result
        }

        isYang() {
            return this.yinYang == "阳"
        }

        private getGanPower(gan: string, wuxing: string) {
            let result = 0
            if (gan != null && wuxing != null) {
                let state = 0
                if (!this.isTianGan) {
                    if (this.muku != null) {
                        if (!this.muku.isOpen) {
                            state = 1
                        } else {
                            state = 2
                        }
                    }
                }
                if (state == 0) {
                    if (wuxing == "木") {
                        result = this.muPower
                    } else if (wuxing == "火") {
                        result = this.huoPower
                    } else if (wuxing == "土") {
                        result = this.tuPower
                    } else if (wuxing == "金") {
                        result = this.jinPower
                    } else if (wuxing == "水") {
                        result = this.shuiPower
                    }
                } else if (state == 1) {
                    if (wuxing == "木") {
                        result = this.biKuMuPower
                    } else if (wuxing == "火") {
                        result = this.biKuHuoPower
                    } else if (wuxing == "土") {
                        result = this.biKuTuPower
                    } else if (wuxing == "金") {
                        result = this.biKuJinPower
                    } else if (wuxing == "水") {
                        result = this.biKuShuiPower
                    }
                } else if (state == 2) {
                    if (wuxing == "木") {
                        result = this.kaiKuMuPower
                    } else if (wuxing == "火") {
                        result = this.kaiKuHuoPower
                    } else if (wuxing == "土") {
                        result = this.kaiKuTuPower
                    } else if (wuxing == "金") {
                        result = this.kaiKuJinPower
                    } else if (wuxing == "水") {
                        result = this.kaiKuShuiPower
                    }
                }
            }
            return result
        }

        getGan1Power() {
            return this.getGanPower(this.cangGan1, this.gan1WuXing)
        }

        getGan2Power() {
            return this.getGanPower(this.cangGan2, this.gan2WuXing)
        }

        getGan3Power() {
            return this.getGanPower(this.cangGan3, this.gan3WuXing)
        }
    }

    export class WangShuaiPower {
        text: string
        wuxing: string
        amount: number
        value: number
        percentage: number
        shishen1: string
        shishenAmount1: number
        shishenValue1: number
        shishenPercentage1: number
        shishenPercentage1InTotal: number
        shishen2: string
        shishenAmount2: number
        shishenValue2: number
        shishenPercentage2: number
        shishenPercentage2InTotal: number
        zengqiang: number
        shishen1ZengQiang: number
        shishen2ZengQiang: number
        shishen1ValueAfterZengQiang: number
        shishen2ValueAfterZengQiang: number
        valueAfterZengQiang: number
    }

    export class PowerInfo {
        wangShuaiPowers: WangShuaiPower[]
        tongdang: number
        yidang: number

        constructor() {
            this.wangShuaiPowers = []
        }
    }

    export class GuanXiInfo {
        shishen: Map<string, string[]>
        tiangan: Map<string, string>
        dizhi: Map<string, string>


        constructor() {
            this.shishen = new Map()
            this.dizhi = new Map()
            this.tiangan = new Map()
        }
    }

    export class ShenSha {
        name: string
        tip: string
        effect: string
    }

    export class ShenShaInfo {
        fenbu: Map<SiZhuWeiZhi, ShenSha[]>

        constructor() {
            this.fenbu = new Map()
        }
    }

    export class GaoNeng {
        arg1: string
        arg2: string
        arg3: string
        guanxi: string
        leibie: string
        text: string
        tip: string
        zhu1: number
        zhu2: number
        zhu3: number
        shishen1: string
        shishen2: string
        shishen3: string
        wuxing1: string
        wuxing2: string
        wuxing3: string
        power1: number
        power2: number
        power3: number
        extraPower1: number
        extraPower2: number
        extraPower3: number
        hehua: string
        fangxiang: string
        key: string
        father: string
        fangxiangTip: string
        banheZengQiang: string
        banheZengQiangWuXing: string
        banheZengQiangShiShen: string
        banheZengQiangPower: number
        gongheZengQiang: string
        gongheZengQiangWuXing: string
        gongheZengQiangShiShen: string
        gongheZengQiangPower: number
        sanhuiZengQiang: string
        sanhuiZengQiangWuXing: string
        sanhuiZengQiangShiShen: number
        sanhuiZengQiangPower: number
        zixingZengQiang: string
        zixingZengQiangWuXing: string
        zixingZengQiangShiShen: string
        zixingZengQiangPower: number
        xingchu1: string
        xingchu2: string
        xingchu1WuXing: string
        xingchu2WuXing: string
        xingchu1ShiShen: string
        xingchu2ShiShen: string
        xingchu1Power: number
        xingchu2Power: number
        xingchong: string
        xingchongArg1: string
        xingchongArg2: string
        xingchongArg1ShiShen: string
        xingchongArg2ShiShen: string
        xingchongArg1Power: number
        xingchongArg2Power: number
        isFromCangGan: boolean
        isFromGongHe: boolean

        private getZhuStr(zhu: number) {
            let result = ""
            switch (zhu) {
                case SiZhuWeiZhi.niangan:
                case SiZhuWeiZhi.nianzhi:
                    result = "年"
                    break
                case SiZhuWeiZhi.yuegan:
                case SiZhuWeiZhi.yuezhi:
                    result = "月"
                    break
                case SiZhuWeiZhi.rigan:
                case SiZhuWeiZhi.rizhi:
                    result = "日"
                    break
                case SiZhuWeiZhi.shigan:
                case SiZhuWeiZhi.shizhi:
                    result = "时"
                    break
            }
            return result
        }

        getTarget1() {
            return this.getZhuStr(this.zhu1)
        }

        getTarget2() {
            return this.getZhuStr(this.zhu2)
        }

        getTarget3() {
            return this.getZhuStr(this.zhu3)
        }

        getShiShen() {
            return this.shishen1 + this.shishen2 + (this.shishen3 != null ? this.shishen3 : "")
        }

        getInfo() {
            let str = ""
            str = buildString(this.text, "   ", this.getTarget1(), this.getTarget2(), this.getTarget3())
            return str
        }
    }

    export class XingGeDesc {
        key: string
        state: string
        low: string
        middle: string
        strong: string
        value: number
        zuozhi: boolean
    }

    export class GeJu {
        key: string
        tip: string
        isChong: boolean
        isChuan: boolean
        isPo: boolean
        isBaoHu: boolean
    }

    export class ZhenJia {
        key: string
        zhenValue: number
        jiaValue: number
        zhenTip: number
        jiaTip: number
    }


    export class ShiShenXiongJi {
        key: string
        isGood: boolean
        eft: string
        solve: string
        tip: string
    }

    export class MangPaiGong {
        he: number = 0
        po: number = 0
        zihe: number = 0
        total: number = 0
        tip: string

        getDesc() {
            return this.total + "层功 (" + this.he + "合 " + this.po + "破 " + this.zihe + "自合)"
        }
    }

    export class GeJuInfo {
        nianlu: GeJu
        jianlu: GeJu
        zhuanlu: GeJu
        guiLu: GeJu
        banlu: GeJu
        zhenjia: ZhenJia[]
        xushi: boolean
        shishen: ShiShenXiongJi[]
        gong: MangPaiGong
        zuozhi: string

        getLuShens(): GeJu[] {
            let lst = []
            if (this.nianlu != null) {
                lst.push(this.nianlu)
            }
            if (this.jianlu != null) {
                lst.push(this.jianlu)
            }
            if (this.zhuanlu != null) {
                lst.push(this.zhuanlu)
            }
            if (this.guiLu != null) {
                lst.push(this.guiLu)
            }
            if (this.banlu != null) {
                lst.push(this.banlu)
            }
            return lst
        }
    }

    function beanToObject(bean: object) {
        let result = {}
        Object.keys(bean).forEach(function (key) {
            result[key] = bean[key]
        })
        return result
    }

    export function copyObjectValues(dst: { [key: string]: any }, src: { [key: string]: any }) {
        Object.keys(dst).forEach(function (key) {
            if (key in src) {
                dst[key] = src[key];
            }
        });
    }

    export function copyArray(src: Array<any>, cls: any, cb: (gnItem: GaoNeng, gnObj: object) => void = null) {
        if (src == null) {
            return null
        }
        let dst = []
        src.forEach(function (value) {
            let obj = new cls()
            copyObjectValues(obj, value)
            cb && cb(obj, value)
            dst.push(obj)
        })
        return dst
    }

    function copyMap(obj: object) {
        let map = new Map()
        Object.keys(obj).forEach(function (key) {
            map.set(key, obj[key])
        })
        return map
    }

    function copyMapForArrayValue(obj: object, cls: any, intKey: boolean) {
        let map = new Map()
        Object.keys(obj).forEach(function (key) {
            let dt = copyArray(obj[key], cls)
            if (intKey) {
                map.set(parseInt(key), dt)
            } else {
                map.set(key, obj[key])
            }

        })
        return map
    }

    function copyMapWithIntKey(obj: object) {
        let map = new Map()
        Object.keys(obj).forEach(function (key) {
            map.set(parseInt(key), obj[key])
        })
        return map
    }

    function copyMuKuFromData(obj: object, key: string, sizhu: SiZhu) {
        let muku = obj[key]["muku"]
        if (muku != null) {
            sizhu.muku = new Models.MuKu()
            copyObjectValues(sizhu.muku, muku)
        }
    }

    function copySiZhu(obj: object) {
        let temp = new SiZhu("", true)
        copyObjectValues(temp, obj)
        temp.gaonengs = copyArray(temp.gaonengs, GaoNeng)
        if (temp.muku != null) {
            let mk = new MuKu()
            copyObjectValues(mk, temp.muku)
            temp.muku = mk
        }
        return temp
    }

    export class JieGou {
        arg1: string
        arg2: string
        guanxi: string
        level: number
        shishen1: string
        shishen2: string
        isTianGan: boolean
        baohu: boolean
    }

    export class JieGouInfo {
        guanHe: JieGou[]
        guanAnHe: JieGou[]
        guanChong: JieGou[]
        guanChuan: JieGou[]
        guanPo: JieGou[]
        shaHe: JieGou[]
        shaAnHe: JieGou[]
        shaChong: JieGou[]
        shaChuan: JieGou[]
        shaPo: JieGou[]
        cai1He: JieGou[]
        cai1AnHe: JieGou[]
        cai1Chong: JieGou[]
        cai1Chuan: JieGou[]
        cai1Po: JieGou[]
        cai2He: JieGou[]
        cai2AnHe: JieGou[]
        cai2Chong: JieGou[]
        cai2Chuan: JieGou[]
        cai2Po: JieGou[]
        yinHe: JieGou[]
        yinAnHe: JieGou[]
        yinChong: JieGou[]
        yinChuan: JieGou[]
        yinPo: JieGou[]
        xiaoHe: JieGou[]
        xiaoAnHe: JieGou[]
        xiaoChong: JieGou[]
        xiaoChuan: JieGou[]
        xiaoPo: JieGou[]
        shiHe: JieGou[]
        shiAnHe: JieGou[]
        shiChong: JieGou[]
        shiChuan: JieGou[]
        shiPo: JieGou[]
        shangHe: JieGou[]
        shangAnHe: JieGou[]
        shangChong: JieGou[]
        shangChuan: JieGou[]
        shangPo: JieGou[]
        biHe: JieGou[]
        biAnHe: JieGou[]
        biChong: JieGou[]
        biChuan: JieGou[]
        biPo: JieGou[]
        jieHe: JieGou[]
        jieAnHe: JieGou[]
        jieChong: JieGou[]
        jieChuan: JieGou[]
        jiePo: JieGou[]
        baohu: Map<string, Boolean>
    }

    export class GaoNengInfo {
        heList: GaoNeng[]
        chongList: GaoNeng[]
        poList: GaoNeng[]
        chuanList: GaoNeng[]
        banHeList: GaoNeng[]
        gongHeList: GaoNeng[]
        anHeList: GaoNeng[]
        sanHuiList: GaoNeng[]
        ziXingList: GaoNeng[]
        xingList: GaoNeng[]
    }

    export class LuckyDayInfo {
        year: number
        month: number
        day: number
        niangan: SiZhu
        yuegan: SiZhu
        rigan: SiZhu
        nianzhi: SiZhu
        yuezhi: SiZhu
        rizhi: SiZhu

        all_zhus() {
            return [this.niangan, this.nianzhi, this.yuegan, this.yuezhi, this.rigan, this.rizhi]
        }
    }

    export class BaZi {
        qiankun: string
        taiyangshi: string
        nianGan!: SiZhu
        nianZhi!: SiZhu
        yueGan!: SiZhu
        yueZhi!: SiZhu
        riGan!: SiZhu
        riZhi!: SiZhu
        shiGan!: SiZhu
        shiZhi!: SiZhu
        sizhuMap: Map<SiZhuWeiZhi, SiZhu>
        chunYin: boolean
        chunYang: boolean

        powerInfo: PowerInfo
        shenshaInfo: ShenShaInfo
        guanxiInfo: GuanXiInfo
        xinggeInfo: Map<string, XingGeDesc[]>
        gejuInfo: GeJuInfo
        gaonengInfo: GaoNengInfo
        jiankang: Map<string, string>
        fangans: SiZhu[]
        jiegouInfo: JieGouInfo
        luckyDays: LuckyDayInfo[]

        tempTianGan: SiZhu
        tempDiZhi: SiZhu
        extraTianGanGaoNengs: Map<number, GaoNeng[]>
        extraDiZhiGaoNengs: Map<number, GaoNeng[]>
        jiegouMap: Map<string, JieGou[][]>

        constructor() {

        }

        getTempTianGan() {
            if (this.tempTianGan == null) {
                this.tempTianGan = new SiZhu('无', true)
                this.tempTianGan.weiZhi = SiZhuWeiZhi.extra
                this.tempTianGan.isTianGan = true
            }
            return this.tempTianGan
        }

        getTempDiZhi() {
            if (this.tempDiZhi == null) {
                this.tempDiZhi = new SiZhu('无', true)
                this.tempDiZhi.weiZhi = SiZhuWeiZhi.extra
                this.tempDiZhi.isTianGan = false
            }
            return this.tempDiZhi
        }

        getExtraGaoNengs(weizhi: number, isTianGan: boolean) {
            let list = []
            if (isTianGan) {
                if (this.extraTianGanGaoNengs != null && this.extraTianGanGaoNengs.has(weizhi)) {
                    list = this.extraTianGanGaoNengs.get(weizhi)
                }
            } else {
                if (this.extraDiZhiGaoNengs != null && this.extraDiZhiGaoNengs.has(weizhi)) {
                    list = this.extraDiZhiGaoNengs.get(weizhi)
                }
            }
            return list
        }

        getFangAn(key: string) {
            let result: SiZhu = null
            if (this.fangans == null) {
                return result
            }
            for (let i = 0; i < this.fangans.length; i++) {
                let fn = this.fangans[i]
                if (fn.text == key) {
                    result = fn
                    break
                }
            }
            return result
        }

        private init() {
            this.sizhuMap = new Map()
            this.sizhuMap.set(SiZhuWeiZhi.niangan, this.nianGan)
            this.sizhuMap.set(SiZhuWeiZhi.yuegan, this.yueGan)
            this.sizhuMap.set(SiZhuWeiZhi.rigan, this.riGan)
            this.sizhuMap.set(SiZhuWeiZhi.shigan, this.shiGan)
            this.sizhuMap.set(SiZhuWeiZhi.nianzhi, this.nianZhi)
            this.sizhuMap.set(SiZhuWeiZhi.yuezhi, this.yueZhi)
            this.sizhuMap.set(SiZhuWeiZhi.rizhi, this.riZhi)
            this.sizhuMap.set(SiZhuWeiZhi.shizhi, this.shiZhi)
            this.chunYin = true
            this.chunYang = true
            let self = this
            this.getSiZhus().forEach(function (sz) {
                if (!sz.isYang()) {
                    self.chunYang = false
                } else {
                    self.chunYin = false
                }
            })
        }


        setExtraGaoNengs(tianganExtra: Map<number, GaoNeng[]>, dizhiExtra: Map<number, GaoNeng[]>) {
            this.extraTianGanGaoNengs = tianganExtra
            this.extraDiZhiGaoNengs = dizhiExtra
        }

        getSiZhus() {
            return [this.nianGan, this.yueGan, this.riGan, this.shiGan, this.nianZhi, this.yueZhi, this.riZhi, this.shiZhi]
        }

        getTexts() {
            let result = []
            this.getSiZhus().forEach(function (sz) {
                result.push(sz.text)
            })
            return result
        }

        setText(weizhi: SiZhuWeiZhi, text: string) {
            let sz = this.sizhuMap.get(weizhi)
            return sz.setText(text)
        }

        getShenSha(weizhi: SiZhuWeiZhi) {
            if (this.shenshaInfo == null || !this.shenshaInfo.fenbu.has(weizhi)) {
                return []
            }
            return this.shenshaInfo.fenbu.get(weizhi)
        }

        getMuKus(): MuKu[] {
            let result = []
            let names = []
            this.getSiZhus().forEach(function (sz) {
                if (sz != null && !sz.isTianGan && sz.muku != null) {
                    if (names.indexOf(sz.muku.text) == -1) {
                        result.push(sz.muku)
                        names.push(sz.muku.text)
                    }
                }
            })
            return result
        }

        static build(text: string): BaZi {
            let dt = new BaZi()
            let obj = JSON.parse(text)
            console.log(obj)
            dt.qiankun = obj.qiankun
            dt.taiyangshi = obj.taiyangshi

            dt.nianGan = new SiZhu(obj.nianGan.text, true)
            copyObjectValues(dt.nianGan, obj.nianGan)
            dt.nianZhi = new SiZhu(obj.nianZhi.text, false)
            copyObjectValues(dt.nianZhi, obj.nianZhi)
            copyMuKuFromData(obj, "nianZhi", dt.nianZhi)

            dt.yueGan = new SiZhu(obj.yueGan.text, true)
            copyObjectValues(dt.yueGan, obj.yueGan)
            dt.yueZhi = new SiZhu(obj.yueZhi.text, false)
            copyObjectValues(dt.yueZhi, obj.yueZhi)
            copyMuKuFromData(obj, "yueZhi", dt.yueZhi)

            dt.riGan = new SiZhu(obj.riGan.text, true)
            copyObjectValues(dt.riGan, obj.riGan)
            dt.riZhi = new SiZhu(obj.riZhi.text, false)
            copyObjectValues(dt.riZhi, obj.riZhi)
            copyMuKuFromData(obj, "riZhi", dt.riZhi)

            dt.shiGan = new SiZhu(obj.shiGan.text, true)
            copyObjectValues(dt.shiGan, obj.shiGan)
            dt.shiZhi = new SiZhu(obj.shiZhi.text, false)
            copyObjectValues(dt.shiZhi, obj.shiZhi)
            copyMuKuFromData(obj, "shiZhi", dt.shiZhi)
            dt.init()

            dt.powerInfo = new PowerInfo()
            dt.powerInfo.tongdang = obj.powerInfo.tongdang
            dt.powerInfo.yidang = obj.powerInfo.yidang
            obj.powerInfo.wangShuaiPowers.forEach(function (wsp) {
                let dst = new WangShuaiPower()
                copyObjectValues(dst, wsp)
                dt.powerInfo.wangShuaiPowers.push(dst)
            })

            dt.guanxiInfo = new GuanXiInfo()
            Object.keys(obj.guanxiInfo.shishen).forEach(function (key) {
                let vs = obj.guanxiInfo.shishen[key]
                let temp = []
                dt.guanxiInfo.shishen.set(key, temp)
                vs.forEach(function (v) {
                    temp.push(v)
                })
            })
            dt.guanxiInfo.tiangan = copyMap(obj.guanxiInfo.tiangan)
            dt.guanxiInfo.dizhi = copyMap(obj.guanxiInfo.dizhi)

            dt.shenshaInfo = new ShenShaInfo()
            Object.keys(obj.shenshaInfo.fenbu).forEach(function (wz) {
                let lst = obj.shenshaInfo.fenbu[wz]
                let temp = []
                lst.forEach(function (item) {
                    let ss = new ShenSha()
                    copyObjectValues(ss, item)
                    temp.push(ss)
                })
                let iWeiZhi = parseInt(wz) as SiZhuWeiZhi
                dt.shenshaInfo.fenbu.set(iWeiZhi, lst)
            })

            dt.xinggeInfo = new Map()
            dt.xinggeInfo.set("wuxing", copyArray(obj.xinggeInfo["wuxing"], XingGeDesc))
            dt.xinggeInfo.set("shishen", copyArray(obj.xinggeInfo["shishen"], XingGeDesc))

            dt.gejuInfo = new GeJuInfo()
            Object.keys(obj.gejuInfo).forEach(function (k) {
                if (k == "zhenjia") {
                    dt.gejuInfo.zhenjia = copyArray(obj.gejuInfo.zhenjia, ZhenJia)
                } else if (k == "xushi") {
                    dt.gejuInfo.xushi = obj.gejuInfo.xushi
                } else if (k == "shishen") {
                    //外面单独处理
                } else if (k == "zuozhi") {
                    dt.gejuInfo.zuozhi = obj.gejuInfo.zuozhi
                } else if (k == "gong") {
                    dt.gejuInfo.gong = new Models.MangPaiGong()
                    copyObjectValues(dt.gejuInfo.gong, obj.gejuInfo.gong)
                } else {
                    dt.gejuInfo[k] = new GeJu()
                    copyObjectValues(dt.gejuInfo[k], obj.gejuInfo[k])
                }
            })
            dt.gejuInfo.shishen = []
            obj.gejuInfo.shishen?.forEach(function (xiongji) {
                let xj = new ShiShenXiongJi()
                xj.eft = xiongji.eft
                xj.key = xiongji.key
                xj.isGood = xiongji.isGood
                xj.solve = xiongji.solve
                xj.tip = xiongji.tip
                dt.gejuInfo.shishen.push(xj)
            })

            dt.gaonengInfo = new GaoNengInfo()
            dt.gaonengInfo.heList = copyArray(obj.gaonengInfo.heList, GaoNeng)
            dt.gaonengInfo.chongList = copyArray(obj.gaonengInfo.chongList, GaoNeng)
            dt.gaonengInfo.poList = copyArray(obj.gaonengInfo.poList, GaoNeng)
            dt.gaonengInfo.chuanList = copyArray(obj.gaonengInfo.chuanList, GaoNeng)
            dt.gaonengInfo.banHeList = copyArray(obj.gaonengInfo.banHeList, GaoNeng)
            dt.gaonengInfo.gongHeList = copyArray(obj.gaonengInfo.gongHeList, GaoNeng)
            dt.gaonengInfo.anHeList = copyArray(obj.gaonengInfo.anHeList, GaoNeng)
            dt.gaonengInfo.sanHuiList = copyArray(obj.gaonengInfo.sanHuiList, GaoNeng)
            dt.gaonengInfo.ziXingList = copyArray(obj.gaonengInfo.ziXingList, GaoNeng)
            dt.gaonengInfo.xingList = copyArray(obj.gaonengInfo.xingList, GaoNeng)

            dt.jiankang = copyMap(obj.jiankang)

            dt.fangans = copyArray(obj.fangans, SiZhu)
            dt.fangans.forEach(function (sz) {
                sz.gaonengs = copyArray(sz.gaonengs, GaoNeng)
                if (sz.extraGaoNengMap != null) {
                    sz.extraGaoNengMap = copyMapForArrayValue(sz.extraGaoNengMap, GaoNeng, true)
                }
                let pj = new PingJia()
                pj.mubiao = copyArray(sz.pingjia.mubiao, Score)
                pj.genji = copyArray(sz.pingjia.genji, Score)
                pj.zhuli = copyArray(sz.pingjia.zhuli, Score)
                pj.mubiaoScore = sz.pingjia.mubiaoScore
                pj.genjiScore = sz.pingjia.genjiScore
                pj.zhuliScore = sz.pingjia.zhuliScore
                pj.shouhuScore = sz.pingjia.shouhuScore
                pj.lushenScore = sz.pingjia.lushenScore
                pj.totalScore = sz.pingjia.totalScore
                sz.pingjia = pj
            })

            dt.jiegouInfo = new JieGouInfo()
            Object.keys(obj.jiegouInfo).forEach(function (key) {
                if (key == "baohu") {
                    dt.jiegouInfo.baohu = copyMap(obj.jiegouInfo[key])
                } else {
                    let lst = obj.jiegouInfo[key]
                    dt.jiegouInfo[key] = copyArray(lst, JieGou)
                }
            })
            dt.jiegouMap = new Map<string, Models.JieGou[][]>()
            let guanLst = [dt.jiegouInfo.guanHe, dt.jiegouInfo.guanAnHe, dt.jiegouInfo.guanChong, dt.jiegouInfo.guanChuan, dt.jiegouInfo.guanPo]
            dt.jiegouMap.set("官", guanLst)
            let shaLst = [dt.jiegouInfo.shaHe, dt.jiegouInfo.shaAnHe, dt.jiegouInfo.shaChong, dt.jiegouInfo.shaChuan, dt.jiegouInfo.shaPo]
            dt.jiegouMap.set("杀", shaLst)
            let cai1Lst = [dt.jiegouInfo.cai1He, dt.jiegouInfo.cai1AnHe, dt.jiegouInfo.cai1Chong, dt.jiegouInfo.cai1Chuan, dt.jiegouInfo.cai1Po]
            dt.jiegouMap.set("财", cai1Lst)
            let cai2Lst = [dt.jiegouInfo.cai2He, dt.jiegouInfo.cai2AnHe, dt.jiegouInfo.cai2Chong, dt.jiegouInfo.cai2Chuan, dt.jiegouInfo.cai2Po]
            dt.jiegouMap.set("才", cai2Lst)
            let yinLst = [dt.jiegouInfo.yinHe, dt.jiegouInfo.yinAnHe, dt.jiegouInfo.yinChong, dt.jiegouInfo.yinChuan, dt.jiegouInfo.yinPo]
            dt.jiegouMap.set("印", yinLst)
            let xiaoLst = [dt.jiegouInfo.xiaoHe, dt.jiegouInfo.xiaoAnHe, dt.jiegouInfo.xiaoChong, dt.jiegouInfo.xiaoChuan, dt.jiegouInfo.xiaoPo]
            dt.jiegouMap.set("枭", xiaoLst)
            let shiLst = [dt.jiegouInfo.shiHe, dt.jiegouInfo.shiAnHe, dt.jiegouInfo.shiChong, dt.jiegouInfo.shiChuan, dt.jiegouInfo.shiPo]
            dt.jiegouMap.set("食", shiLst)
            let shangLst = [dt.jiegouInfo.shangHe, dt.jiegouInfo.shangAnHe, dt.jiegouInfo.shangChong, dt.jiegouInfo.shangChuan, dt.jiegouInfo.shangPo]
            dt.jiegouMap.set("伤", shangLst)
            let biLst = [dt.jiegouInfo.biHe, dt.jiegouInfo.biAnHe, dt.jiegouInfo.biChong, dt.jiegouInfo.biChuan, dt.jiegouInfo.biPo]
            dt.jiegouMap.set("比", biLst)
            let jieLst = [dt.jiegouInfo.jieHe, dt.jiegouInfo.jieAnHe, dt.jiegouInfo.jieChong, dt.jiegouInfo.jieChuan, dt.jiegouInfo.jiePo]
            dt.jiegouMap.set("劫", jieLst)

            dt.luckyDays = copyArray(obj.luckyDays, LuckyDayInfo)
            dt.luckyDays.forEach(function (luck) {
                luck.niangan = copySiZhu(luck.niangan)
                luck.nianzhi = copySiZhu(luck.nianzhi)
                luck.yuegan = copySiZhu(luck.yuegan)
                luck.yuezhi = copySiZhu(luck.yuezhi)
                luck.rigan = copySiZhu(luck.rigan)
                luck.rizhi = copySiZhu(luck.rizhi)
            })

            //TODO end build
            return dt
        }
    }

}