import {YiXueLogic} from "./YiXueLogic.ts";
import {ElMessage} from "element-plus";
import {h} from "vue";
import Cookies from "js-cookie";

export namespace Tools {
    export function getColorByWuXing(wuxing: string) {
        let result = "white"
        if (wuxing == null) {
            wuxing = "土"
        }

        switch (wuxing) {
            case "木":
                result = "#13ce66"
                break
            case "火":
                result = "#F56C6C"
                break
            case "土":
                result = "#7a7273"
                break
            case "金":
                result = "#E6A23C"
                break
            case "水":
                result = "#409EFF"
                break
        }
        return result
    }

    export function getTypeByGuanXi(guanxi: string) {
        let result = "primary"
        if (guanxi == "刑" || guanxi == "冲" || guanxi == "破" || guanxi == "自刑") {
            result = "warning"
        } else if (guanxi == "穿") {
            result = "danger"
        } else if (guanxi == "合" || guanxi == "暗合") {
            result = "success"
        }
        return result
    }

    export function getTypeByText(text: string) {
        let result = "primary"
        if (text.endsWith("刑") || text.endsWith("冲") || text.endsWith("破") || text.endsWith("自刑")) {
            result = "warning"
        } else if (text.endsWith("穿")) {
            result = "danger"
        } else if (text.endsWith("合") || text.endsWith("暗合")) {
            result = "success"
        }
        return result
    }

    export function getTypeByWuXing(wuxing: string) {
        switch (wuxing) {
            case YiXueLogic.WuXing.Mu:
                return "success"
            case YiXueLogic.WuXing.Huo:
                return "danger"
            case YiXueLogic.WuXing.Tu:
                return "info"
            case YiXueLogic.WuXing.Jin:
                return "warning"
            case YiXueLogic.WuXing.Shui:
                return "primary"
        }
        return "info"
    }

    export function getShiShenFullName(name: string) {
        switch (name) {
            case "官":
                return "正官"
            case "杀":
                return "七杀"
            case "比":
                return "比肩"
            case "劫":
                return "劫财"
            case "食":
                return "食神"
            case "伤":
                return "伤官"
            case "财":
                return "正财"
            case "才":
                return "偏财"
            case "印":
                return "正印"
            case "枭":
                return "偏印"
        }
    }

    export function copyToClipboard(str: string) {
        navigator.clipboard.writeText(str)
    }

    export function showMessage(msg: string) {
        ElMessage({
            message: h('p', null, [
                h('i', {style: 'color: black'}, msg),
            ]),
        })
    }

    export function getToken() {
        return Cookies.get("userToken")
    }
}