import {Net} from "./Net.ts";
import {Models} from "./Models.ts";
import {DataCenter} from "./DataCenter.ts";

export namespace YiXueLogic {


    export enum WuXing {
        Mu = "木",
        Huo = "火",
        Tu = "土",
        Jin = "金",
        Shui = "水",
        UnKnown = ""
    }

    export enum YinYang {
        Yin = "阴",
        Yang = "阳"
    }

    export const tiangans = ["甲", "乙", "丙", "丁", "戊", "己", "庚", "辛", "壬", "癸"]
    export const dizhis = ["子", "丑", "寅", "卯", "辰", "巳", "午", "未", "申", "酉", "戌", "亥"]

    let serverUrl = "https://yixue.cxhy.cn"
    let localUrl = "http://127.0.0.1:8000"
    //"https://yixueapi.cxhy.cn"
    //"http://127.0.0.1:8000"

    function getServerUrl() {
        if (DataCenter.getDebugMode()) {
            return "http://127.0.0.1:8000/api"
        } else {
            return "https://yixue.cxhy.cn/api"
        }
    }

    function getRandomInt(min: number, max: number): number {
        min = Math.ceil(min);
        max = Math.floor(max);
        return Math.floor(Math.random() * (max - min + 1)) + min;
    }

    export function randomOfTianGan() {
        let len = tiangans.length
        let index = getRandomInt(0, len - 1)
        return tiangans[index]
    }

    export function randomOfDiZhi() {
        let len = dizhis.length
        let index = getRandomInt(0, len - 1)
        return dizhis[index]
    }

    export function getSiZhuInfo(request: Models.BaZiRequest, cb: (bazi: Models.BaZi) => void) {
        Net.post(getServerUrl() + "/getSiZhuInfo", request.toData(), function (response) {
            if (response != null) {
                let bazi = Models.BaZi.build(response)
                DataCenter.getBaZiRequest().texts = bazi.getTexts()
                cb && cb(bazi)
            } else {
                cb && cb(null)
            }
        })
    }

    export function customLogin(request: Models.CustomLogin, cb: (name: string, sexy: number) => void) {
        Net.post(getServerUrl() + "/customLogin", request.toData(), function (response) {
            if (response != null) {
                let obj = JSON.parse(response)
                cb && cb(obj.name, obj.sexy)
            } else {
                cb && cb(null, null)
            }
        })
    }

    export function saveUser(user: Models.SaveUser, cb: (state: number, msg: string) => void) {
        Net.post(getServerUrl() + "/saveUser", user.toData(), function (response) {
            if (response != null) {
                let obj = JSON.parse(response)
                cb && cb(obj.state, obj.msg)
            } else {
                cb && cb(-1, null)
            }
        })
    }

    export function queryUser(customer: string, filter: string, cb: (users: Models.SaveUser[]) => void) {
        let query = new Models.QueryUser()
        query.customer = customer
        if (filter == null) {
            filter = ""
        }
        query.filter = filter
        Net.post(getServerUrl() + "/queryUser", query.toData(), function (response) {
            if (response != null) {
                let obj = JSON.parse(response)
                cb && cb(Models.copyArray(obj, Models.SaveUser))
            } else {
                cb && cb(null)
            }
        })
    }

    export function deleteUser(id: number, customer: string, cb: (users: Models.SaveUser[]) => void) {
        let del = new Models.DeleteUser()
        del.customer = customer
        del.id = id
        Net.post(getServerUrl() + "/deleteUser", del.toData(), function (response) {
            if (response != null) {
                let obj = JSON.parse(response)
                cb && cb(Models.copyArray(obj, Models.SaveUser))
            } else {
                cb && cb(null)
            }
        })
    }

    export function test() {
        Net.post(getServerUrl() + "/test", {}, function (response) {
            if (response != null) {
                let obj = JSON.parse(response)
                console.log(obj)
            } else {
                console.log("error")
            }
        })
    }
}